@@ -27,14 +27,12 @@
 #			Bug reported and fixed by Frank Boer (Thanks!)
 #			- pause eating cpu cycles
 #			- no sound output in silent mode
+# 0.5	2010-08-13	Added some features mentioned in the gui thread in maemo.talk
+#			removed Stop button - Now Player stops in pause mode but remembers the position
+#			Prev button now jumps to beginning of song if duration is more than 10s
 #
 # TODO:
 #
-# * save position in song/audio book after shutdown/sleep
-# * remove STOP button. Instead use Play/Pause. Pause will completly stop
-#   playback but remembers position in song and resumes to this. Currently
-#   Pause sets gstream to wait mode
-#
 # * rearange buttons in portrait mode
 # * do a backround rescan every startup (seperate thread) and then update playlist
 # * option to manually delete files on the device (long press in directory browser?)
@@ -59,8 +57,8 @@ import gst
 import hildon
 from portrait import FremantleRotation
 
-version = "0.4 - 2010-04-11"
-# Enable debugging
+version = "0.5 - 2010-08-14"
+# Enable/disable debugging
 dbg = 0
 
 class GTK_Main:
@@ -75,8 +73,7 @@ class GTK_Main:
     self.act_file_id = 0	# id of above
     self.act_file_len = 0	# len in nano-seconds of above
 
-    self.pause_pos = 0;
-    self.pause_flag = 0;
+    self.act_pos = 0;
 
     self.clear_tags()
     
@@ -94,18 +91,18 @@ class GTK_Main:
         self.prefs = pickle.load(fd) # overwrite default values
         fd.close()
       except:
-        if (dbg): print "Can't load preferences"
-    self.act_file_id = self.prefs["lastfile"]    
+        if (dbg): print "Can't load settings"
 
     # Look, if the preferences are new enough
     # If not (or some tag "version" just don't exists), overwrite with default
     try:
-      if self.prefs["version"] < 4:
+      if self.prefs["version"] < 5:
         self.init_prefs()
         self.save_prefs()    
     except:
       self.init_prefs()
       self.save_prefs()
+
     
     if self.prefs["bigbuttons"] == 1: 
       self.std_size = gtk.HILDON_SIZE_AUTO|gtk.HILDON_SIZE_THUMB_HEIGHT
@@ -161,7 +158,7 @@ class GTK_Main:
     menu = hildon.AppMenu()
 
     self.pref_btn = hildon.GtkButton(self.std_size)
-    self.pref_btn.set_label("Preferences");
+    self.pref_btn.set_label("Settings");
     self.pref_btn.connect("clicked",self.show_pref_win_cb)
     menu.append(self.pref_btn)
 
@@ -251,8 +248,10 @@ class GTK_Main:
     self.pbar_eb.add(self.pbar)
     vbox.pack_start(self.pbar_eb, False, True, 0)
     self.pbar.show()
-    #self.play_name = self.pl_files[self.act_file_id][0]
-    self.pbar.set_text(self.pl_files[self.act_file_id][0]+" - 00:00 / 00:00")
+    self.act_file_id = self.prefs["lastfile"]      
+    self.play_name = self.pl_files[self.act_file_id][0]
+    self.act_pos = self.prefs["lastpos"]
+    self.update_pbar_s(self.prefs["lastpos"],self.prefs["lastdur"])
     
     buttonbox = gtk.HBox()
     prev_btn = self.new_button(gtk.STOCK_MEDIA_PREVIOUS)
@@ -270,12 +269,12 @@ class GTK_Main:
 
     self.play_btn = hildon.GtkButton(self.std_size)
     self.play_btn.add(self.play_img)
-    self.play_btn.connect("clicked", self.play_pause_cb)
+    self.play_btn.connect("clicked", self.play_stop_cb)
     buttonbox.add(self.play_btn)
 
-    stop_btn = self.new_button(gtk.STOCK_MEDIA_STOP)
-    stop_btn.connect("clicked", self.stop_cb)
-    buttonbox.add(stop_btn)
+    #stop_btn = self.new_button(gtk.STOCK_MEDIA_STOP)
+    #stop_btn.connect("clicked", self.stop_cb)
+    #buttonbox.add(stop_btn)
 
     forward_btn = self.new_button(gtk.STOCK_MEDIA_FORWARD)
     forward_btn.connect("clicked", self.forward_cb)
@@ -288,10 +287,12 @@ class GTK_Main:
     #self.lbl_time = gtk.Label()
     #self.lbl_time.set_text("00:00 / 00:00")
     #buttonbox.add(self.lbl_time)
-		
+
     vbox.pack_start(buttonbox,False,True)	
-    self.window.set_title("Easy Player")
+    self.window.set_title("EasyPlayer")
     self.window.show_all()
+
+    self.window.connect("delete_event", self.delete_event)
 		
     self.player = gst.element_factory_make("playbin2", "player")
     fakesink = gst.element_factory_make("fakesink", "fakesink")
@@ -302,28 +303,67 @@ class GTK_Main:
     self.running = False;
     self.time_format = gst.Format(gst.FORMAT_TIME)
     if self.prefs["resumeplay"] == 1:
-      self.play_pause_h()
+      self.play_stop_h()
     self.timeout = 0;
     self.reset_timer() # Reset TimeOut Timer
     self.rot_obj = FremantleRotation("EasyPlayer",self.window,"1.0",FremantleRotation.AUTOMATIC)
 
+  # 'delete_event' is called when the window is to be closed
+  # save preferences file
+  def delete_event(self, widget, event, data=None):
+    self.save_prefs()
+    if dbg: print("delete event")
+    return False # 'false' destroys Main Window
+
+  # update progress bar in nano seconds
+  #
+  def update_pbar_ns(self, pos_int, dur_int):
+    self.update_pbar_s(pos_int/1000000000, dur_int/1000000000)
+  
+  # update progress bar in seconds
+  #
+  def update_pbar_s(self, pos_int, dur_int):
+    pos_str = self.convert_s(pos_int)
+    dur_str = self.convert_s(dur_int)
+
+    try:
+      dt = self.prefs["displaytimeout"]
+      to = self.timeout
+    except:
+      dt = 0
+      to = -1
+
+    if (to == -1) or (dt == 0):
+      self.pbar.set_text(self.play_name + " - " + pos_str + " / " + dur_str)
+    else:
+      to_str = self.convert_s(self.timeout)   
+      self.pbar.set_text(self.play_name + " - " + pos_str + " / " + dur_str + " (" + to_str +")")
+    if (dur_int > 0) and (pos_int > 0):  
+      self.pbar.set_fraction(float(pos_int) / float(dur_int))
+
   # Thread to display time / update process bar while running
   # copied from pygst tutorial
   def play_thread(self):
     play_thread_id = self.play_thread_id
     oldtime = time.time()
     gtk.gdk.threads_enter()
-    if not self.pause_flag:
-      self.reset_bar()
     gtk.gdk.threads_leave()
     while play_thread_id == self.play_thread_id:
       try:
         time.sleep(0.2)
+        if dbg: print ("Actual pos:",self.act_pos)
+        if self.act_pos > 0:
+          seek_ns = self.act_pos * 1000000000
+          self.player.seek_simple(self.time_format, gst.SEEK_FLAG_FLUSH, seek_ns) 
+          time.sleep(0.2)
+          if (self.player.query_position(self.time_format, None)[0] < seek_ns):
+            time.sleep(0.4)
+            self.player.seek_simple(self.time_format, gst.SEEK_FLAG_FLUSH, seek_ns) 
         dur_int = self.player.query_duration(self.time_format, None)[0]
+        self.prefs["lastdur"] = dur_int / 1000000000 
 	self.act_file_len = dur_int
-        dur_str = self.convert_ns(dur_int)
         gtk.gdk.threads_enter()
-	self.pbar.set_text(self.play_name + " - 00:00 / " + dur_str)
+	self.update_pbar_ns(self.act_pos*1000000000,dur_int)
         gtk.gdk.threads_leave()
 	break
       except:
@@ -333,46 +373,31 @@ class GTK_Main:
     # Main Thread for displaying time infos etc.
     #
     while play_thread_id == self.play_thread_id:
-      if not self.pause_flag:
-        try: 
-          pos_int = self.player.query_position(self.time_format, None)[0]
-          pos_str = self.convert_ns(pos_int)
-          
-          # Do we use timeout for suspend?
-          if self.timeout <> -1:
-            if self.timeout > 0: 
-              newtime = time.time()
-              self.timeout = self.timeout - int(newtime-oldtime)
-              oldtime = newtime
-            else: # Timeout is reached, suspend player
-              self.stop_reset() # prepare stopping of threat
-              gtk.gdk.threads_enter()
-              self.reset_bar()
-              gtk.gdk.threads_leave()
-              self.player.set_state(gst.STATE_NULL)       
-              break; 	# leave threat
-              
-          if self.prefs["metatags"] == 1: # show metatags instead of filename
-            if self.tags["title"] <> "":
-              self.play_name = self.tags["artist"] + " - " + self.tags["title"]                          
-               
-          if play_thread_id == self.play_thread_id:
-            gtk.gdk.threads_enter()
-            # don't display time to suspend if we don't use timeout
-            # or we disabled displaying it
-            if (self.timeout == -1) or (self.prefs["displaytimeout"] == 0):
-              self.pbar.set_text(self.play_name + " - " + pos_str + " / " + dur_str)
-            else:
-              to_str = self.convert_s(self.timeout)   
-              self.pbar.set_text(self.play_name + " - " + pos_str + " / " + dur_str + " (" + to_str +")")
-            if dur_int <> 0:  
-              self.pbar.set_fraction(float(pos_int) / float(dur_int))
-            gtk.gdk.threads_leave()
-          time.sleep(1)
-        except:
-          time.sleep(0.2)
-      else:
-        time.sleep(1) # Sleep for one second in pause mode. 
+      try: 
+        pos_int = self.player.query_position(self.time_format, None)[0]
+        self.prefs["lastpos"] = pos_int / 1000000000 
+        # Do we use timeout for suspend?
+        if self.timeout <> -1:
+          if self.timeout > 0: 
+            newtime = time.time()
+            self.timeout = self.timeout - int(newtime-oldtime)
+            oldtime = newtime
+          else: # Timeout is reached, suspend player
+            self.save_prefs()
+            self.stop_player() # prepare stopping of threat
+            break; 	# leave threat
+            
+        if self.prefs["metatags"] == 1: # show metatags instead of filename
+          if self.tags["title"] <> "":
+            self.play_name = self.tags["artist"] + " - " + self.tags["title"]                          
+             
+        if play_thread_id == self.play_thread_id:
+          gtk.gdk.threads_enter()
+          self.update_pbar_ns(pos_int,dur_int)
+          gtk.gdk.threads_leave()
+        time.sleep(1)
+      except:
+        time.sleep(0.2)
 
   # Extracs the parent directory name of a given path
   #
@@ -409,12 +434,14 @@ class GTK_Main:
   #
   def menu_rescan_cb(self, btn, lbl):
     # clear fileslist
+    hildon.hildon_banner_show_information(self.window, "", "Scanning for new audio files...")
     self.pl_dirs = []      # list to store the directorys containing music files
     self.dc = 0          # actual number of directory
     self.pl_files = []     # list to store filenames of music files
     self.max_files = 0
     self.rescan(self.base_media_dir)
     self.save_playlist()
+    hildon.hildon_banner_show_information(self.window, "", "Restart Easyplayer to get new playlist!")
 
   # row acivated callback is activated if a row is
   # double clicked by the user
@@ -427,9 +454,7 @@ class GTK_Main:
     id = self.get_id(value[0])
     #print "Path: ",path," Value:",value," id:",id
     if id <> None:
-      self.stop_h() # Stop current file if one is playing
-      self.act_file_id = id
-      self.play_pause_h() # Start new file  
+      self.play_new_file(id) # Start new file at pos 0
 
   # Get the file_id to a given name
   #
@@ -449,22 +474,6 @@ class GTK_Main:
     button.add(image)
     return button
 
-  # Set process bar to 0 and time/text to 00:00
-  #
-  def reset_bar(self):
-    self.pbar.set_text(self.play_name + " - 00:00 / 00:00")
-    self.pbar.set_fraction(0.0)
-
-  # Set process bar to 0 and time/text to 00:00
-  #
-  def update_bar(self, pos, len, frac):
-    if (self.timeout == -1) or (self.prefs["displaytimeout"] == 0):
-      self.pbar.set_text(self.playname + " - " + pos + " / " + len)
-    else:
-      to_str = self.convert_s(self.timeout)   
-      self.pbar.set_text(self.playname + " - " + pos + " / " + len + " ("+to_str+")")
-    self.pbar.set_fraction(frac)
-	
   # Menu "About" callback - display About...
   #
   def menu_about_cb(self, btn, lbl):
@@ -476,7 +485,7 @@ class GTK_Main:
     #if response == gtk.RESPONSE_DELETE_EVENT:
     #  print "show_information_note: gtk.RESPONSE_DELETE_EVENT"
 
-  # Menu "About" callback - display About...
+  # Menu "Display Tags" callback - display mp3 tags of actual playing file
   #
   def menu_display_tags_cb(self, btn, lbl):
     note = hildon.hildon_note_new_information(self.window, \
@@ -511,10 +520,12 @@ class GTK_Main:
   def pbar_eb_press_cb(self, widget, event):
     if event.type == gtk.gdk.BUTTON_PRESS:
       frac = event.x/float(widget.get_allocation().width)
-      self.player.seek_simple(self.time_format, gst.SEEK_FLAG_FLUSH, 
-		int(self.act_file_len*frac))
-    self.pbar.set_fraction(frac)
-    # FIXME - update progressbar now -> put it in an own function	
+      if (self.running == True):
+        self.player.seek_simple(self.time_format, gst.SEEK_FLAG_FLUSH, int(self.act_file_len*frac))
+        self.pbar.set_fraction(frac)
+      else:
+        self.act_pos = int(self.act_file_len*frac)/1000000000 # to round to 1 sec
+	self.update_pbar_ns(self.act_pos*1000000000,self.act_file_len)
 
   # Set Image of Button Play to "Play"
   #  
@@ -522,26 +533,30 @@ class GTK_Main:
     self.play_img.set_from_stock(gtk.STOCK_MEDIA_PLAY, gtk.ICON_SIZE_BUTTON)
     self.play_img.show()
 
-  # Set Image of Button Play to "Pause"
+  # Set Image of Button Play to "Stop"
   #  
-  def set_pause_img(self):
-    self.play_img.set_from_stock(gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_BUTTON)
+  def set_stop_img(self):
+    self.play_img.set_from_stock(gtk.STOCK_MEDIA_STOP, gtk.ICON_SIZE_BUTTON)
     self.play_img.show()
+
+  # Resets file position to 0 and starts new file
+  # 
+  def play_new_file(self, id):
+    self.stop_player() 		# Stop current file if one is playing
+    self.act_file_id = id	# Select new id
+    self.act_pos = 0		# Start at pos 0
+    self.play_stop_h()  	# let's go...
 		
-  # Start / Pause Playback
+  # Start / Stop Playback
   #
-  def play_pause_cb(self, w):
+  def play_stop_cb(self, w):
     self.reset_timer() # Reset TimeOut Timer
-    self.play_pause_h()
+    self.play_stop_h()
   
-  # Start / Pause Handler
+  # Start / Stop Handler
   #
-  def play_pause_h(self):
-    if self.pause_flag == True:
-        self.set_pause_img()
-        self.player.set_state(gst.STATE_PLAYING)
-        self.pause_flag = None
-    elif self.running == False:
+  def play_stop_h(self):
+    if self.running == False: 
       file_id = self.act_file_id; #which file to play?
       dir_id = self.pl_files[file_id][1]
       self.play_name = self.pl_files[file_id][0]
@@ -549,7 +564,7 @@ class GTK_Main:
       self.clear_tags()
       #print filepath
       if os.path.isfile(filepath):
-        self.set_pause_img()
+        self.set_stop_img()
         self.player.set_property("uri", "file://" + filepath)
         self.player.set_state(gst.STATE_PLAYING)
 	# Start thread
@@ -558,35 +573,21 @@ class GTK_Main:
         self.prefs["lastfile"] = file_id
         self.save_prefs()
     else:
-      # We are running, go to pause mode
-      #self.pause_pos = self.player.query_position(self.time_format, None)[0]
-      self.pause_flag = True          
-      self.player.set_state(gst.STATE_PAUSED)
-      self.set_play_img()
-      # don't self.reset_bar()
-      #self.stop_reset()      
-
-  # "Stop" Button callback - Stop Playback
-  #
-  def stop_cb(self, w):
-    self.reset_timer() # Reset TimeOut Time
-    self.stop_h()
+      # We are running, so stop player
+      self.act_pos = self.player.query_position(self.time_format, None)[0]/1000000000
+      if dbg: print("Stopping player: act_pos=",self.act_pos)
+      self.stop_player()
+      self.prefs["lastpos"] = self.act_pos
+      self.save_prefs()
 
   # "Stop" Handler
-  #
-  def stop_h(self):
+  # 
+  def stop_player(self):
     if self.running == True:
+      self.set_play_img()
+      self.running = False
+      self.play_thread_id = None
       self.player.set_state(gst.STATE_NULL)
-      self.reset_bar()
-      self.stop_reset() # stop thread	
-      self.pause_flag = 0;
-      
-  # stop_reset - restore old button
-  #
-  def stop_reset(self):
-    self.play_thread_id = None
-    self.set_play_img()
-    self.running = False
 
   # Parse any messages from gstreamer
   #    
@@ -594,23 +595,18 @@ class GTK_Main:
     t = message.type
     if t == gst.MESSAGE_EOS:
       if self.act_file_id < self.max_files: # Is there at least one file more to play
-        self.stop_h() 		# Stop current file if one is playing
-        self.act_file_id += 1
-        #self.act_file_id = self.act_file_id + 1
-        self.play_pause_h() 	# Start new file  
+        self.play_new_file(self.act_file_id+1) 	# Start new file  
       else: 			# No more files, just stop
-        self.stop_h()
+        self.stop_player()
       
     elif t == gst.MESSAGE_ERROR:
-      self.stop_h()
+      self.stop_player()
       err, debug = message.parse_error()
       if (dbg): print "Error: %s" % err, debug
     elif t == gst.MESSAGE_TAG:
       for key in message.parse_tag().keys():
         self.tags[key] = message.structure[key]
         #print "Tags:",key,message.structure[key]
-    #else:
-    #  print "Msg %d" % t 
 
   # Skip backward 10 s callback
   #
@@ -632,20 +628,18 @@ class GTK_Main:
   #
   def prev_cb(self, w):
     self.reset_timer() # Reset TimeOut Timer
-    if self.act_file_id > 0: # Is there at least one previous file to play
-      self.stop_h() # Stop current file if one is playing
-      self.act_file_id = self.act_file_id - 1
-      self.play_pause_h() # Start new file  
+    pos_int = self.player.query_position(self.time_format, None)[0]
+    if pos_int > 10*1000000000: # Jump back to beginning of song
+      self.player.seek_simple(self.time_format, gst.SEEK_FLAG_FLUSH, 0)
+    elif self.act_file_id > 0: # Is there at least one previous file to play
+      self.play_new_file(self.act_file_id-1) # Start new file  
       
   # Select next file to play
   #
   def next_cb(self, w):
     self.reset_timer() # Reset TimeOut Timer
     if self.act_file_id < self.max_files: # Is there at least one file more to play
-      self.stop_h() # Stop current file if one is playing
-      self.act_file_id += 1
-      #self.act_file_id = self.act_file_id + 1
-      self.play_pause_h() # Start new file  
+      self.play_new_file(self.act_file_id+1) # Start new file  
 
   # Convert a time in secondes to min:sec
   # taken from the pygst tutorial 
@@ -684,39 +678,41 @@ class GTK_Main:
   def show_pref_win_cb(self, widget):
     # Create the main window
     prefwin = hildon.StackableWindow()
-    prefwin.set_title("Preferences")
+    prefwin.set_title("Settings")
    
     vbox = gtk.VBox(False,0)
     prefwin.add(vbox)
  
-    cb1 = hildon.CheckButton(self.std_size)
-    cb1.set_label("Display remaining time until timeout")
-    cb1.set_active(self.prefs["displaytimeout"])
-    cb1.connect("toggled", self.cb_toggled_timeout)
+    pref_size = gtk.HILDON_SIZE_AUTO|gtk.HILDON_SIZE_FINGER_HEIGHT
+ 
+    cb1 = hildon.CheckButton(pref_size)
+    cb1.set_label("Resume playing last file after startup")
+    cb1.set_active(self.prefs["resumeplay"])
+    cb1.connect("toggled", self.cb_toggled_resume)
     vbox.pack_start(cb1, False, True)
 
-    cb2 = hildon.CheckButton(self.std_size)
-    cb2.set_label("Resume playing last file after startup")
-    cb2.set_active(self.prefs["resumeplay"])
-    cb2.connect("toggled", self.cb_toggled_resume)
+    cb2 = hildon.CheckButton(pref_size)
+    cb2.set_label("Use big (thumb) buttons")
+    cb2.set_active(self.prefs["bigbuttons"])
+    cb2.connect("toggled", self.cb_toggled_big_btn)
     vbox.pack_start(cb2, False, True)
-
-    cb3 = hildon.CheckButton(self.std_size)
-    cb3.set_label("Use big buttons")
-    cb3.set_active(self.prefs["bigbuttons"])
-    cb3.connect("toggled", self.cb_toggled_big_btn)
-    vbox.pack_start(cb3, False, True)
     
-    cb4 = hildon.CheckButton(self.std_size)
-    cb4.set_label("Output sound in silent mode")
-    cb4.set_active(self.prefs["silenthack"])
-    cb4.connect("toggled", self.cb_toggled_silent_hack)
+    cb3 = hildon.CheckButton(pref_size)
+    cb3.set_label("Output sound in silent mode")
+    cb3.set_active(self.prefs["silenthack"])
+    cb3.connect("toggled", self.cb_toggled_silent_hack)
+    vbox.pack_start(cb3, False, True)
+
+    cb4 = hildon.CheckButton(pref_size)
+    cb4.set_label("Display Metatags")
+    cb4.set_active(self.prefs["metatags"])
+    cb4.connect("toggled", self.cb_toggled_metatags)
     vbox.pack_start(cb4, False, True)
 
-    cb5 = hildon.CheckButton(self.std_size)
-    cb5.set_label("Display Metatags")
-    cb5.set_active(self.prefs["metatags"])
-    cb5.connect("toggled", self.cb_toggled_metatags)
+    cb5 = hildon.CheckButton(pref_size)
+    cb5.set_label("Display remaining time until timeout")
+    cb5.set_active(self.prefs["displaytimeout"])
+    cb5.connect("toggled", self.cb_toggled_timeout)
     vbox.pack_start(cb5, False, True)
 
     timeouts = [ "5 min", "10 min", "20 min", "30 min", "60 min", "Never" ]
@@ -755,12 +751,15 @@ class GTK_Main:
   def cb_toggled_big_btn(self, cb):
     self.prefs["bigbuttons"] = cb.get_active()
     self.save_prefs()
+    hildon.hildon_banner_show_information(self.window, "", "Restart Easyplayer to see this feature")
 
   # Callback for changing beviour with silent profile
   #
   def cb_toggled_silent_hack(self, cb):
     self.prefs["silenthack"] = cb.get_active()
     self.save_prefs()
+    hildon.hildon_banner_show_information(self.window, "", "Restart Easyplayer to see this feature")
+
     # Changing name once started does not work. Restart player for this to work.
     #if cb.get_active():
     #    gtk.set_application_name("FMRadio")
@@ -793,9 +792,10 @@ class GTK_Main:
   # 
   def init_prefs(self):  
     self.prefs = {
-      "version":4,		# Preferences version 0.4
+      "version":5,		# Preferences version 0.4
       "lastfile":0,		# id of last played file
-      "lastpos":0,		# position in seconds
+      "lastpos":0,		# position of last played file in seconds
+      "lastdur":0,		# duration of last played file
       "hideext":0,		# Hide filename extention, eg. ".mp3",".ogg"
       "displaytimeout":1,	# Show the remaining time until timeout
       "resumeplay":0,		# resume playing last file
@@ -836,7 +836,7 @@ class GTK_Main:
       if (dbg):print "Save dirs..."
       fd = open(self.pl_dirs_file,"w")
       pickle.dump(self.pl_dirs, fd)
-      fd.close()
+      fd.close()      
       # Save filenames
       if (dbg):print "Save filenames..."
       fd = open(self.pl_files_file,"w")
